@@ -1,18 +1,16 @@
-from __future__ import absolute_import, unicode_literals
-
 import logging
+import urllib
+
+from pykka.messages import ProxyCall
 
 from mopidy.audio import PlaybackState
-from mopidy.compat import urllib
 from mopidy.core import listener
 from mopidy.internal import deprecation, models, validation
 
 logger = logging.getLogger(__name__)
 
 
-class PlaybackController(object):
-    pykka_traversable = True
-
+class PlaybackController:
     def __init__(self, audio, backends, core):
         # TODO: these should be internal
         self.backends = backends
@@ -34,7 +32,8 @@ class PlaybackController(object):
 
         if self._audio:
             self._audio.set_about_to_finish_callback(
-                self._on_about_to_finish_callback)
+                self._on_about_to_finish_callback
+            )
 
     def _get_backend(self, tl_track):
         if tl_track is None:
@@ -42,8 +41,6 @@ class PlaybackController(object):
         uri_scheme = urllib.parse.urlparse(tl_track.track.uri).scheme
         return self.backends.with_playback.get(uri_scheme, None)
 
-    # Properties
-
     def get_current_tl_track(self):
         """Get the currently playing or selected track.
 
@@ -58,12 +55,6 @@ class PlaybackController(object):
         """
         self._current_tl_track = value
 
-    current_tl_track = deprecation.deprecated_property(get_current_tl_track)
-    """
-    .. deprecated:: 1.0
-        Use :meth:`get_current_tl_track` instead.
-    """
-
     def get_current_track(self):
         """
         Get the currently playing or selected track.
@@ -72,13 +63,7 @@ class PlaybackController(object):
 
         Returns a :class:`mopidy.models.Track` or :class:`None`.
         """
-        return getattr(self.get_current_tl_track(), 'track', None)
-
-    current_track = deprecation.deprecated_property(get_current_track)
-    """
-    .. deprecated:: 1.0
-        Use :meth:`get_current_track` instead.
-    """
+        return getattr(self.get_current_tl_track(), "track", None)
 
     def get_current_tlid(self):
         """
@@ -90,7 +75,7 @@ class PlaybackController(object):
 
         .. versionadded:: 1.1
         """
-        return getattr(self.get_current_tl_track(), 'tlid', None)
+        return getattr(self.get_current_tl_track(), "tlid", None)
 
     def get_stream_title(self):
         """Get the current stream title or :class:`None`."""
@@ -121,16 +106,10 @@ class PlaybackController(object):
         validation.check_choice(new_state, validation.PLAYBACK_STATES)
 
         (old_state, self._state) = (self.get_state(), new_state)
-        logger.debug('Changing state: %s -> %s', old_state, new_state)
+        logger.debug("Changing state: %s -> %s", old_state, new_state)
 
         self._trigger_playback_state_changed(old_state, new_state)
 
-    state = deprecation.deprecated_property(get_state, set_state)
-    """
-    .. deprecated:: 1.0
-        Use :meth:`get_state` and :meth:`set_state` instead.
-    """
-
     def get_time_position(self):
         """Get time position in milliseconds."""
         if self._pending_position is not None:
@@ -142,68 +121,6 @@ class PlaybackController(object):
         else:
             return 0
 
-    time_position = deprecation.deprecated_property(get_time_position)
-    """
-    .. deprecated:: 1.0
-        Use :meth:`get_time_position` instead.
-    """
-
-    def get_volume(self):
-        """
-        .. deprecated:: 1.0
-            Use :meth:`core.mixer.get_volume()
-            <mopidy.core.MixerController.get_volume>` instead.
-        """
-        deprecation.warn('core.playback.get_volume')
-        return self.core.mixer.get_volume()
-
-    def set_volume(self, volume):
-        """
-        .. deprecated:: 1.0
-            Use :meth:`core.mixer.set_volume()
-            <mopidy.core.MixerController.set_volume>` instead.
-        """
-        deprecation.warn('core.playback.set_volume')
-        return self.core.mixer.set_volume(volume)
-
-    volume = deprecation.deprecated_property(get_volume, set_volume)
-    """
-    .. deprecated:: 1.0
-        Use :meth:`core.mixer.get_volume()
-        <mopidy.core.MixerController.get_volume>` and
-        :meth:`core.mixer.set_volume()
-        <mopidy.core.MixerController.set_volume>` instead.
-    """
-
-    def get_mute(self):
-        """
-        .. deprecated:: 1.0
-            Use :meth:`core.mixer.get_mute()
-            <mopidy.core.MixerController.get_mute>` instead.
-        """
-        deprecation.warn('core.playback.get_mute')
-        return self.core.mixer.get_mute()
-
-    def set_mute(self, mute):
-        """
-        .. deprecated:: 1.0
-            Use :meth:`core.mixer.set_mute()
-            <mopidy.core.MixerController.set_mute>` instead.
-        """
-        deprecation.warn('core.playback.set_mute')
-        return self.core.mixer.set_mute(mute)
-
-    mute = deprecation.deprecated_property(get_mute, set_mute)
-    """
-    .. deprecated:: 1.0
-        Use :meth:`core.mixer.get_mute()
-        <mopidy.core.MixerController.get_mute>` and
-        :meth:`core.mixer.set_mute()
-        <mopidy.core.MixerController.set_mute>` instead.
-    """
-
-    # Methods
-
     def _on_end_of_stream(self):
         self.set_state(PlaybackState.STOPPED)
         if self._current_tl_track:
@@ -254,10 +171,13 @@ class PlaybackController(object):
         there is no unsafe access of state in core. This must block until
         we get a response.
         """
-        self.core.actor_ref.ask({
-            'command': 'pykka_call', 'args': tuple(), 'kwargs': {},
-            'attr_path': ('playback', '_on_about_to_finish'),
-        })
+        self.core.actor_ref.ask(
+            ProxyCall(
+                attr_path=["playback", "_on_about_to_finish"],
+                args=[],
+                kwargs={},
+            )
+        )
 
     def _on_about_to_finish(self):
         if self._state == PlaybackState.STOPPED:
@@ -282,14 +202,16 @@ class PlaybackController(object):
                         self._pending_tl_track = pending
                         break
                 except Exception:
-                    logger.exception('%s backend caused an exception.',
-                                     backend.actor_ref.actor_class.__name__)
+                    logger.exception(
+                        "%s backend caused an exception.",
+                        backend.actor_ref.actor_class.__name__,
+                    )
 
             self.core.tracklist._mark_unplayable(pending)
             pending = self.core.tracklist.eot_track(pending)
             count -= 1
             if not count:
-                logger.info('No playable track in the list.')
+                logger.info("No playable track in the list.")
                 break
 
     def _on_tracklist_change(self):
@@ -298,10 +220,11 @@ class PlaybackController(object):
 
         Used by :class:`mopidy.core.TracklistController`.
         """
-        if not self.core.tracklist.tl_tracks:
+        tl_tracks = self.core.tracklist.get_tl_tracks()
+        if not tl_tracks:
             self.stop()
             self._set_current_tl_track(None)
-        elif self.get_current_tl_track() not in self.core.tracklist.tl_tracks:
+        elif self.get_current_tl_track() not in tl_tracks:
             self._set_current_tl_track(None)
 
     def next(self):
@@ -329,7 +252,7 @@ class PlaybackController(object):
             current = pending
             count -= 1
             if not count:
-                logger.info('No playable track in the list.')
+                logger.info("No playable track in the list.")
                 break
 
         # TODO return result?
@@ -352,6 +275,9 @@ class PlaybackController(object):
 
         Note that the track **must** already be in the tracklist.
 
+        .. deprecated:: 3.0
+            The ``tl_track`` argument. Use ``tlid`` instead.
+
         :param tl_track: track to play
         :type tl_track: :class:`mopidy.models.TlTrack` or :class:`None`
         :param tlid: TLID of the track to play
@@ -364,7 +290,7 @@ class PlaybackController(object):
         tlid is None or validation.check_integer(tlid, min=1)
 
         if tl_track:
-            deprecation.warn('core.playback.play:tl_track_kwarg', pending=True)
+            deprecation.warn("core.playback.play:tl_track_kwarg")
 
         if tl_track is None and tlid is not None:
             for tl_track in self.core.tracklist.get_tl_tracks():
@@ -395,7 +321,7 @@ class PlaybackController(object):
             pending = self.core.tracklist.next_track(current)
             count -= 1
             if not count:
-                logger.info('No playable track in the list.')
+                logger.info("No playable track in the list.")
                 break
 
         # TODO return result?
@@ -423,8 +349,10 @@ class PlaybackController(object):
             if not backend.playback.change_track(pending_tl_track.track).get():
                 return False
         except Exception:
-            logger.exception('%s backend caused an exception.',
-                             backend.actor_ref.actor_class.__name__)
+            logger.exception(
+                "%s backend caused an exception.",
+                backend.actor_ref.actor_class.__name__,
+            )
             return False
 
         # TODO: Wrap backend calls in error handling.
@@ -434,8 +362,11 @@ class PlaybackController(object):
             except TypeError:
                 # TODO: check by binding against underlying play method using
                 # inspect and otherwise re-raise?
-                logger.error('%s needs to be updated to work with this '
-                             'version of Mopidy.', backend)
+                logger.error(
+                    "%s needs to be updated to work with this "
+                    "version of Mopidy.",
+                    backend,
+                )
                 return False
         elif state == PlaybackState.PAUSED:
             return backend.playback.pause().get()
@@ -445,7 +376,7 @@ class PlaybackController(object):
             self._pending_tl_track = None
             return True
 
-        raise Exception('Unknown state: %s' % state)
+        raise Exception(f"Unknown state: {state}")
 
     def previous(self):
         """
@@ -473,7 +404,7 @@ class PlaybackController(object):
             current = pending
             count -= 1
             if not count:
-                logger.info('No playable track in the list.')
+                logger.info("No playable track in the list.")
                 break
 
         # TODO: no return value?
@@ -504,11 +435,10 @@ class PlaybackController(object):
         validation.check_integer(time_position)
 
         if time_position < 0:
-            logger.debug(
-                'Client seeked to negative position. Seeking to zero.')
+            logger.debug("Client seeked to negative position. Seeking to zero.")
             time_position = 0
 
-        if not self.core.tracklist.tracks:
+        if not self.core.tracklist.get_length():
             return False
 
         if self.get_state() == PlaybackState.STOPPED:
@@ -554,39 +484,41 @@ class PlaybackController(object):
                 self.set_state(PlaybackState.STOPPED)
 
     def _trigger_track_playback_paused(self):
-        logger.debug('Triggering track playback paused event')
-        if self.current_track is None:
+        logger.debug("Triggering track playback paused event")
+        if self.get_current_tl_track() is None:
             return
         listener.CoreListener.send(
-            'track_playback_paused',
+            "track_playback_paused",
             tl_track=self.get_current_tl_track(),
-            time_position=self.get_time_position())
+            time_position=self.get_time_position(),
+        )
 
     def _trigger_track_playback_resumed(self):
-        logger.debug('Triggering track playback resumed event')
-        if self.current_track is None:
+        logger.debug("Triggering track playback resumed event")
+        if self.get_current_tl_track() is None:
             return
         listener.CoreListener.send(
-            'track_playback_resumed',
+            "track_playback_resumed",
             tl_track=self.get_current_tl_track(),
-            time_position=self.get_time_position())
+            time_position=self.get_time_position(),
+        )
 
     def _trigger_track_playback_started(self):
         if self.get_current_tl_track() is None:
             return
 
-        logger.debug('Triggering track playback started event')
+        logger.debug("Triggering track playback started event")
         tl_track = self.get_current_tl_track()
         self.core.tracklist._mark_playing(tl_track)
         self.core.history._add_track(tl_track.track)
-        listener.CoreListener.send('track_playback_started', tl_track=tl_track)
+        listener.CoreListener.send("track_playback_started", tl_track=tl_track)
 
     def _trigger_track_playback_ended(self, time_position_before_stop):
         tl_track = self.get_current_tl_track()
         if tl_track is None:
             return
 
-        logger.debug('Triggering track playback ended event')
+        logger.debug("Triggering track playback ended event")
 
         if not self._previous:
             self.core.tracklist._mark_played(self._current_tl_track)
@@ -594,29 +526,31 @@ class PlaybackController(object):
 
         # TODO: Use the lowest of track duration and position.
         listener.CoreListener.send(
-            'track_playback_ended',
+            "track_playback_ended",
             tl_track=tl_track,
-            time_position=time_position_before_stop)
+            time_position=time_position_before_stop,
+        )
 
     def _trigger_playback_state_changed(self, old_state, new_state):
-        logger.debug('Triggering playback state change event')
+        logger.debug("Triggering playback state change event")
         listener.CoreListener.send(
-            'playback_state_changed',
-            old_state=old_state, new_state=new_state)
+            "playback_state_changed", old_state=old_state, new_state=new_state
+        )
 
     def _trigger_seeked(self, time_position):
         # TODO: Trigger this from audio events?
-        logger.debug('Triggering seeked event')
-        listener.CoreListener.send('seeked', time_position=time_position)
+        logger.debug("Triggering seeked event")
+        listener.CoreListener.send("seeked", time_position=time_position)
 
     def _save_state(self):
         return models.PlaybackState(
             tlid=self.get_current_tlid(),
             time_position=self.get_time_position(),
-            state=self.get_state())
+            state=self.get_state(),
+        )
 
     def _load_state(self, state, coverage):
-        if state and 'play-last' in coverage and state.tlid is not None:
+        if state and "play-last" in coverage and state.tlid is not None:
             if state.state == PlaybackState.PAUSED:
                 self._start_paused = True
             if state.state in (PlaybackState.PLAYING, PlaybackState.PAUSED):
